from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from collections import OrderedDict
import logging
from tripal.client import Client

logging.getLogger("requests").setLevel(logging.CRITICAL)
log = logging.getLogger()


class DbClient(Client):
    CLIENT_BASE = '/tripal_api/'

    def get_dbs(self, db_id=None, name=None):
        """
        Get all dbs

        :type db_id: str
        :param db_id: A db ID

        :type name: str
        :param name: filter on db name

        :rtype: list of dict
        :return: Dbs information
        """

        dbs = self._request('chado/list', {'table': 'db'})

        if db_id:
            dbs = [v for v in dbs if v['db_id'] == str(db_id)]

        return dbs

    def get_mviews(self, name=None):
        """
        Get all materialized views

        :type name: str
        :param name: filter on mview name

        :rtype: list of dict
        :return: materialized views information
        """

        data = {
            'table': 'tripal_mviews',
        }

        mvs = self._request('chado/list', data)

        if name:
            for m in mvs:
                if m['name'] == name:
                    return m

        return mvs

    def index(self, mode="website", table=None, index_name=None, queues=10, fields=[], links={}, tokenizer='standard', job_name=None, no_wait=False):
        """
        Schedule database indexing using elasticsearch

        :type mode: str
        :param mode: Indexing mode: 'website' to index everything, 'table' to index a single table (default: website)

        :type table: str
        :param table: Table to index (only in 'table' mode)

        :type index_name: str
        :param index_name: Index name (only in 'table' mode)

        :type queues: int
        :param queues: Number of indexing task queues

        :type fields: list of str
        :param fields: Fields to index (only in 'table' mode), syntax: <field_name>|<field_type>, field_type should be one of 'string', 'keyword', 'date', 'long', 'double', 'boolean', 'ip', 'object', 'nested', 'geo_point', 'geo_shape', or 'completion'

        :type links: list of str
        :param links: List of links to show to users, syntax: <column-where-to-show-the-link>|</your/url/[any-column-name]>

        :type tokenizer: str
        :param tokenizer: Tokenizer to use (one of 'standard', 'letter', 'lowercase', 'whitespace', 'uax_url_email', 'classic', 'ngram', 'edge_ngram', 'keyword', 'pattern', or 'path_hierarchy'; default='standard')

        :type job_name: str
        :param job_name: Name of the job

        :type no_wait: bool
        :param no_wait: Do not wait for job to complete

        :rtype: str
        :return: Indexing information
        """

        modes = {
            'table': 'table',
            'website': 'website',
        }

        if mode not in modes:
            raise Exception("Mode should be 'table' or 'website'")

        if mode == 'table' and not index_name:
            raise Exception("index_name is required in 'table' mode")

        tokenizers = {
            'standard': 'standard',
            'letter': 'letter',
            'lowercase': 'lowercase',
            'whitespace': 'whitespace',
            'uax_url_email': 'uax_url_email',
            'classic': 'classic',
            'ngram': 'ngram',
            'edge_ngram': 'edge_ngram',
            'keyword': 'keyword',
            'pattern': 'pattern',
            'path_hierarchy': 'path_hierarchy',
        }

        if tokenizer not in tokenizers:
            raise Exception("Unknown tokenizer")

        if mode not in mode:
            raise Exception("Mode should be 'table' or 'website'")

        fields_real = {}
        for f in fields:
            fs = f.split('|', 1)
            fields_real[fs[0]] = fs[1]

        links_real = {}
        for l in links:
            ls = l.split('|', 1)
            if ls[0] not in fields_real:
                raise Exception("Cannot add a link to column '%s' because it is not indexed" % ls[0])
            links_real[ls[0]] = ls[1]

        data = {
            'mode': mode,
            'table': table,
            'index_name': index_name,
            'queues': queues,
            'tokenizer': tokenizer,
            'fields': fields_real,
            'links': links_real,
        }

        res = self._request('elasticsearch/index', data)

        if res and 'status' in res and res['status'] == 'error':
            raise Exception("Failed to schedule indexing, error: %s" % res['errors'])

        if not job_name:
            job_name = 'Elasticsearch indexing'

        job_args = OrderedDict()
        job_args[0] = 10
        job_args[1] = 120

        r = self.tripal.job.add_job(job_name, 'tripal_rest_api', 'tripal_rest_api_run_indexing', job_args)
        if 'job_id' not in r or not r['job_id']:
            raise Exception("Failed to create job, received %s" % r)

        if no_wait:
            return r
        else:
            return self._run_job_and_wait(r['job_id'])

        return res

    def populate_mviews(self, name=None, no_wait=None):
        """
        Populate materialized views

        :type name: str
        :param name: filter on mview name

        :type no_wait: bool
        :param no_wait: Do not wait for job to complete

        :rtype: str
        :return: Loading information
        """

        if name:
            mview_id = self.get_mviews(name)['mview_id']

            self._do_populate_mview(name, mview_id, no_wait)
        else:
            mviews = self.get_mviews()

            for m in mviews:
                self._do_populate_mview(m)

    def _do_populate_mview(self, mview, no_wait=False, job_name=None):
        if not job_name:
            job_name = 'Populate materialized views \'%s\'' % mview['name']

        print('Populating view \'%s\'' % mview['name'])

        job_args = OrderedDict()
        job_args[0] = mview['mview_id']

        r = self.tripal.job.add_job(job_name, 'tripal_core', 'tripal_populate_mview', job_args)
        if 'job_id' not in r or not r['job_id']:
            raise Exception("Failed to create job, received %s" % r)

        if no_wait:
            return r
        else:
            return self._run_job_and_wait(r['job_id'])



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, str_output


@click.command('load_blast')
@click.argument("name", type=str)
@click.argument("program", type=str)
@click.argument("programversion", type=str)
@click.argument("sourcename", type=str)
@click.argument("blast_output", type=str)
@click.option(
    "--blast_ext",
    help="If looking for files in a directory, extension of the blast result files",
    type=str
)
@click.option(
    "--blastdb",
    help="Name of the database blasted against (must be in the Chado db table)",
    type=str
)
@click.option(
    "--blastdb_id",
    help="ID of the database blasted against (must be in the Chado db table)",
    type=str
)
@click.option(
    "--blast_parameters",
    help="Blast parameters used to produce these results",
    type=str
)
@click.option(
    "--query_re",
    help="The regular expression that can uniquely identify the query name. This parameters is required if the feature name is not the first word in the blast query name.",
    type=str
)
@click.option(
    "--query_type",
    help="The feature type (e.g. 'gene', 'mRNA', 'contig') of the query. It must be a valid Sequence Ontology term.",
    type=str
)
@click.option(
    "--query_uniquename",
    help="Use this if the --query-re regular expression matches unique names instead of names in the database.",
    is_flag=True
)
@click.option(
    "--is_concat",
    help="If the blast result file is simply a list of concatenated blast results.",
    is_flag=True
)
@click.option(
    "--search_keywords",
    help="Extract keywords for Tripal search",
    is_flag=True
)
@click.option(
    "--no_wait",
    help="Do not wait for job to complete",
    is_flag=True
)
@click.option(
    "--algorithm",
    help="analysis algorithm",
    type=str
)
@click.option(
    "--sourceversion",
    help="analysis sourceversion",
    type=str
)
@click.option(
    "--sourceuri",
    help="analysis sourceuri",
    type=str
)
@click.option(
    "--description",
    help="analysis description",
    type=str
)
@click.option(
    "--date_executed",
    help="analysis date_executed (yyyy-mm-dd)",
    type=str
)
@pass_context
@custom_exception
@str_output
def cli(ctx, name, program, programversion, sourcename, blast_output, blast_ext="", blastdb="", blastdb_id="", blast_parameters="", query_re="", query_type="", query_uniquename=False, is_concat=False, search_keywords=False, no_wait=False, algorithm="", sourceversion="", sourceuri="", description="", date_executed=""):
    """Create a Blast analysis

Output:

    Loading information
    """
    return ctx.gi.analysis.load_blast(name, program, programversion, sourcename, blast_output, blast_ext=blast_ext, blastdb=blastdb, blastdb_id=blastdb_id, blast_parameters=blast_parameters, query_re=query_re, query_type=query_type, query_uniquename=query_uniquename, is_concat=is_concat, search_keywords=search_keywords, no_wait=no_wait, algorithm=algorithm, sourceversion=sourceversion, sourceuri=sourceuri, description=description, date_executed=date_executed)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, list_output


@click.command('get_mviews')
@click.option(
    "--name",
    help="filter on mview name",
    type=str
)
@pass_context
@custom_exception
@list_output
def cli(ctx, name=""):
    """Get all materialized views

Output:

    materialized views information
    """
    return ctx.gi.db.get_mviews(name=name)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, list_output


@click.command('get_analysis_nodes')
@click.option(
    "--node",
    help="filter on node id",
    type=int
)
@pass_context
@custom_exception
@list_output
def cli(ctx, node=""):
    """Get analysis nodes

Output:

    Analysis node information
    """
    return ctx.gi.analysis.get_analysis_nodes(node=node)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, list_output


@click.command('get_dbs')
@click.option(
    "--db_id",
    help="A db ID",
    type=str
)
@click.option(
    "--name",
    help="filter on db name",
    type=str
)
@pass_context
@custom_exception
@list_output
def cli(ctx, db_id="", name=""):
    """Get all dbs

Output:

    Dbs information
    """
    return ctx.gi.db.get_dbs(db_id=db_id, name=name)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, dict_output


@click.command('get_logs')
@click.argument("stdout", type=str)
@click.argument("stderr", type=str)
@pass_context
@custom_exception
@dict_output
def cli(ctx, stdout, stderr):
    """Get job output

Output:

    Output information
    """
    return ctx.gi.job.get_logs(stdout, stderr)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from collections import OrderedDict
import logging
from tripal.client import Client

logging.getLogger("requests").setLevel(logging.CRITICAL)
log = logging.getLogger()


class FeatureClient(Client):
    CLIENT_BASE = '/tripal_api/'

    def sync(self, organism=None, organism_id=None, max_sync='', types=[], ids=[],
             job_name=None, no_wait=None):
        """
        Synchronize some features

        :type organism: str
        :param organism: Common name of the organism to sync

        :type organism_id: str
        :param organism_id: ID of the organism to sync

        :type max_sync: str
        :param max_sync: Maximum number of features to sync (default: all)

        :type types: list of str
        :param types: List of types of records to be synced (e.g. gene mRNA, default: all)

        :type ids: list of str
        :param ids: List of names of records to be synced (e.g. gene0001, default: all)

        :type job_name: str
        :param job_name: Name of the job

        :type no_wait: bool
        :param no_wait: Return immediately without waiting for job completion

        :rtype: str
        :return: status
        """

        if organism_id:
            found_org = self.tripal.organism.get_organisms(organism_id=organism_id)
            if not found_org:
                raise Exception("Invalid organism ID")
        elif organism:
            found_org = self.tripal.organism.get_organisms(common=organism)
            if not found_org:
                found_org = self.tripal.organism.get_organisms(abbr=organism)

            if not found_org:
                raise Exception("Invalid organism name")

            organism_id = found_org[0]['organism_id']

        if not organism_id:
            raise Exception("Either organism or organism_id is required")

        if not job_name:
            job_name = 'Sync Features'

        job_args = OrderedDict()
        job_args['base_table'] = 'feature'
        job_args['max_sync'] = max_sync
        job_args['organism_id'] = organism_id
        job_args['types'] = types
        job_args['ids'] = ids
        job_args['linking_table'] = 'chado_feature'
        job_args['node_type'] = 'chado_feature'

        r = self.tripal.job.add_job(job_name, 'chado_feature', 'chado_node_sync_records', job_args)
        if 'job_id' not in r or not r['job_id']:
            raise Exception("Failed to create job, received %s" % r)

        if no_wait:
            return r
        else:
            return self._run_job_and_wait(r['job_id'])



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, dict_output


@click.command('add_organism')
@click.argument("genus", type=str)
@click.argument("common", type=str)
@click.argument("abbr", type=str)
@click.option(
    "--species",
    help="The species of the organism",
    type=str
)
@click.option(
    "--comment",
    help="A comment / description",
    type=str
)
@click.option(
    "--infraspecific_rank",
    help="The type name of infraspecific name for any taxon below the rank of species. Must be one of ['subspecies', 'varietas', 'subvariety', 'forma', 'subforma']",
    type=str
)
@click.option(
    "--infraspecific_name",
    help="The infraspecific name for this organism.",
    type=str
)
@pass_context
@custom_exception
@dict_output
def cli(ctx, genus, common, abbr, species="", comment="", infraspecific_rank="", infraspecific_name=""):
    """Add a new organism to the database

Output:

    Organism information
    """
    return ctx.gi.organism.add_organism(genus, common, abbr, species=species, comment=comment, infraspecific_rank=infraspecific_rank, infraspecific_name=infraspecific_name)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import json
import logging
import time
from tripal.client import Client

logging.getLogger("requests").setLevel(logging.CRITICAL)
log = logging.getLogger()


class JobClient(Client):
    CLIENT_BASE = '/tripal_api/'

    def get_jobs(self, job_id=None):
        """
        Get all jobs

        :type job_id: int
        :param job_id: job id

        :rtype: list of dict
        :return: Jobs information
        """

        if job_id:
            return [self._get('job/%s' % job_id, {})]
        else:
            return self._get('job', {})

    def add_job(self, name, module, callback, arguments, priority=10):
        """
        Schedule a new job

        :type name: str
        :param name: The name of the job

        :type module: str
        :param module: The Tripal module name to invoke

        :type callback: str
        :param callback: The Tripal module callback function to invoke

        :type arguments: str
        :param arguments: A JSON string representing an array of arguments (e.g. "['some', 'arg', 42, 'foo']")

        :type priority: int
        :param priority: An integer score to prioritize the job

        :rtype: dict
        :return: Job information
        """

        # Ensure we will send an array whatever the input is
        if not isinstance(arguments, list) and not isinstance(arguments, dict):
            arguments = json.loads(arguments)

        data = {
            'name': name,
            'modulename': module,
            'callback': callback,
            'arguments': arguments,
            'priority': priority,
        }

        return self._request('job', data)

    def run_jobs(self, wait=True):
        """
        Run jobs in queue. There is no way to trigger a single job execution.

        :type wait: bool
        :param wait: Wait for job completion

        :rtype: dict
        :return: Job information
        """

        res = None
        while res is None or res['status'] == 'busy':
            res = self._request('job/run', {})
            if res['status'] == 'busy':
                if not wait:
                    return res
                time.sleep(20)

        return res

    def wait(self, job_id):
        """
        Wait for a job completion

        :type job_id: int
        :param job_id: job id

        :rtype: dict
        :return: Job information
        """

        job = None
        while not job or job['status'] not in ('Completed', 'Cancelled', 'Error'):
            # First call is_running to make sure the job status is updated in case it
            # exited in a wrong way
            self._check_running()

            job = self.get_jobs(job_id)
            if not job:
                raise Exception("Could not find job %s" % job_id)
            job = job[0]
            time.sleep(20)

        return job

    def _check_running(self):

        return self._request('job/is_running', {})

    def get_logs(self, stdout, stderr):
        """
        Get job output

        :type stdout: str
        :param stdout: Path to stdout file, as returned by run_jobs

        :type stderr: str
        :param stderr: Path to stderr file, as returned by run_jobs

        :rtype: dict
        :return: Output information
        """

        res = self._request('job/logs', {'stdout': stdout, 'stderr': stderr})

        res['stdout'] = json.loads(res['stdout'])
        res['stderr'] = json.loads(res['stderr'])

        return res



import click
from tripaille.commands.analysis.add_analysis import cli as func0
from tripaille.commands.analysis.get_analyses import cli as func1
from tripaille.commands.analysis.get_analysis_nodes import cli as func2
from tripaille.commands.analysis.load_blast import cli as func3
from tripaille.commands.analysis.load_fasta import cli as func4
from tripaille.commands.analysis.load_gff3 import cli as func5
from tripaille.commands.analysis.load_go import cli as func6
from tripaille.commands.analysis.load_interpro import cli as func7
from tripaille.commands.analysis.sync import cli as func8


@click.group()
def cli():
    pass


cli.add_command(func0)
cli.add_command(func1)
cli.add_command(func2)
cli.add_command(func3)
cli.add_command(func4)
cli.add_command(func5)
cli.add_command(func6)
cli.add_command(func7)
cli.add_command(func8)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, dict_output


@click.command('add_job')
@click.argument("name", type=str)
@click.argument("module", type=str)
@click.argument("callback", type=str)
@click.argument("arguments", type=str)
@click.option(
    "--priority",
    help="An integer score to prioritize the job",
    default="10",
    show_default=True,
    type=int
)
@pass_context
@custom_exception
@dict_output
def cli(ctx, name, module, callback, arguments, priority=10):
    """Schedule a new job

Output:

    Job information
    """
    return ctx.gi.job.add_job(name, module, callback, arguments, priority=priority)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, dict_output


@click.command('wait')
@click.argument("job_id", type=int)
@pass_context
@custom_exception
@dict_output
def cli(ctx, job_id):
    """Wait for a job completion

Output:

    Job information
    """
    return ctx.gi.job.wait(job_id)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, list_output


@click.command('get_jobs')
@click.option(
    "--job_id",
    help="job id",
    type=int
)
@pass_context
@custom_exception
@list_output
def cli(ctx, job_id=""):
    """Get all jobs

Output:

    Jobs information
    """
    return ctx.gi.job.get_jobs(job_id=job_id)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, list_output


@click.command('get_analyses')
@click.option(
    "--analysis_id",
    help="An analysis ID",
    type=str
)
@click.option(
    "--name",
    help="analysis name",
    type=str
)
@click.option(
    "--program",
    help="analysis program",
    type=str
)
@click.option(
    "--programversion",
    help="analysis programversion",
    type=str
)
@click.option(
    "--algorithm",
    help="analysis algorithm",
    type=str
)
@click.option(
    "--sourcename",
    help="analysis sourcename",
    type=str
)
@click.option(
    "--sourceversion",
    help="analysis sourceversion",
    type=str
)
@click.option(
    "--sourceuri",
    help="analysis sourceuri",
    type=str
)
@click.option(
    "--date_executed",
    help="analysis date_executed (yyyy-mm-dd)",
    type=str
)
@pass_context
@custom_exception
@list_output
def cli(ctx, analysis_id="", name="", program="", programversion="", algorithm="", sourcename="", sourceversion="", sourceuri="", date_executed=""):
    """Get analyses

Output:

    Analysis information
    """
    return ctx.gi.analysis.get_analyses(analysis_id=analysis_id, name=name, program=program, programversion=programversion, algorithm=algorithm, sourcename=sourcename, sourceversion=sourceversion, sourceuri=sourceuri, date_executed=date_executed)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, str_output


@click.command('sync')
@click.option(
    "--organism",
    help="Common name of the organism to sync",
    type=str
)
@click.option(
    "--organism_id",
    help="ID of the organism to sync",
    type=str
)
@click.option(
    "--max_sync",
    help="Maximum number of features to sync (default: all)",
    type=str
)
@click.option(
    "--types",
    help="List of types of records to be synced (e.g. gene mRNA, default: all)",
    type=str,
    multiple=True
)
@click.option(
    "--ids",
    help="List of names of records to be synced (e.g. gene0001, default: all)",
    type=str,
    multiple=True
)
@click.option(
    "--job_name",
    help="Name of the job",
    type=str
)
@click.option(
    "--no_wait",
    help="Return immediately without waiting for job completion",
    is_flag=True
)
@pass_context
@custom_exception
@str_output
def cli(ctx, organism="", organism_id="", max_sync="", types=None, ids=None, job_name="", no_wait=""):
    """Synchronize some features

Output:

    status
    """
    return ctx.gi.feature.sync(organism=organism, organism_id=organism_id, max_sync=max_sync, types=types, ids=ids, job_name=job_name, no_wait=no_wait)



"""Base tripal client
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import click
import json
import requests

from future import standard_library
standard_library.install_aliases()


class Client(object):
    """
    Base client class implementing methods to make queries to the server
    """

    def __init__(self, tripalinstance, **requestArgs):
        self.tripal = tripalinstance

        self.__verify = requestArgs.get('verify', True)
        self._requestArgs = requestArgs
        self._session = None

        if 'verify' in self._requestArgs:
            del self._requestArgs['verify']

    def _get_csrf_token(self):

        url = self.tripal.tripal_url + self.CLIENT_BASE

        headers = {
            'Content-Type': 'application/json'
        }

        if self._session:
            headers['Cookie'] = self._session

        auth = None
        if self.tripal.auth_login and self.tripal.auth_password:
            auth = (self.tripal.auth_login, self.tripal.auth_password)

        # First get a CSRF token
        r = requests.post(url + 'user/token.json', headers=headers,
                          verify=self.__verify, auth=auth, **self._requestArgs)

        if r.status_code == 200:
            d = r.json()
            return d['token']
        else:
            # @see self.body for HTTP response body
            raise Exception("Unexpected response from tripal while getting CSRF token %s: %s" %
                            (r.status_code, r.text))

    def _login(self):

        # First get a CSRF token
        csrfToken = self._get_csrf_token()

        # Then login
        url = self.tripal.tripal_url + self.CLIENT_BASE

        headers = {
            'Content-Type': 'application/json',
            'X-CSRF-Token': csrfToken
        }

        data = {
            'username': self.tripal.username,
            'password': self.tripal.password,
        }

        auth = None
        if self.tripal.auth_login and self.tripal.auth_password:
            auth = (self.tripal.auth_login, self.tripal.auth_password)

        r = requests.post(url + 'user/login.json', data=json.dumps(data), headers=headers,
                          verify=self.__verify, auth=auth, **self._requestArgs)

        if r.status_code == 200:
            d = r.json()
            self._session = d['session_name'] + '=' + d['sessid']
        else:
            # @see self.body for HTTP response body
            raise Exception("Unexpected response from tripal while logging in %s: %s" %
                            (r.status_code, r.text))

    def _request(self, clientMethod, data, post_params={}):
        url = self.tripal.tripal_url + self.CLIENT_BASE + clientMethod

        if not self._session:
            self._login()

        # Get a CSRF token
        csrfToken = self._get_csrf_token()

        headers = {
            'Content-Type': 'application/json',
            'Cookie': self._session,
            'X-CSRF-Token': csrfToken
        }

        auth = None
        if self.tripal.auth_login and self.tripal.auth_password:
            auth = (self.tripal.auth_login, self.tripal.auth_password)

        r = requests.post(url, data=json.dumps(data), headers=headers,
                          verify=self.__verify, auth=auth, params=post_params, **self._requestArgs)

        if r.status_code == 200:
            d = r.json()
            return d

        # @see self.body for HTTP response body
        raise Exception("Unexpected response from tripal %s: %s" %
                        (r.status_code, r.text))

    def _get(self, clientMethod, get_params):
        url = self.tripal.tripal_url + self.CLIENT_BASE + clientMethod

        if not self._session:
            self._login()

        headers = {
            'Content-Type': 'application/json',
            'Cookie': self._session  # "SESSc9c711a015d1f2624abf5bff20b25337=FQ4HGEDbfXVEeU9YTGEQMTHyE-VsspsdaNVxhFsVN0k"
        }

        auth = None
        if self.tripal.auth_login and self.tripal.auth_password:
            auth = (self.tripal.auth_login, self.tripal.auth_password)

        r = requests.get(url, headers=headers, verify=self.__verify,
                         params=get_params, auth=auth, **self._requestArgs)

        if r.status_code == 200:
            d = r.json()
            return d

        # @see self.body for HTTP response body
        raise Exception("Unexpected response from tripal %s: %s" %
                        (r.status_code, r.text))

    def _run_job_and_wait(self, job_id):
        run_res = self.tripal.job.run_jobs()
        self.tripal.job.wait(job_id)

        logs = self.tripal.job.get_logs(run_res['stdout'], run_res['stderr'])

        if logs['status'] == 'ok':
            print(logs['stdout'])
            click.echo(click.style(logs['stderr'], bold=True, fg='red'), err=True)
            return ''
        else:
            return logs



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, dict_output


@click.command('add_analysis')
@click.argument("name", type=str)
@click.argument("program", type=str)
@click.argument("programversion", type=str)
@click.argument("sourcename", type=str)
@click.option(
    "--algorithm",
    help="analysis algorithm",
    type=str
)
@click.option(
    "--sourceversion",
    help="analysis sourceversion",
    type=str
)
@click.option(
    "--sourceuri",
    help="analysis sourceuri",
    type=str
)
@click.option(
    "--description",
    help="analysis description",
    type=str
)
@click.option(
    "--date_executed",
    help="analysis date_executed (yyyy-mm-dd)",
    type=str
)
@pass_context
@custom_exception
@dict_output
def cli(ctx, name, program, programversion, sourcename, algorithm="", sourceversion="", sourceuri="", description="", date_executed=""):
    """Create an analysis

Output:

    Analysis information
    """
    return ctx.gi.analysis.add_analysis(name, program, programversion, sourcename, algorithm=algorithm, sourceversion=sourceversion, sourceuri=sourceuri, description=description, date_executed=date_executed)



# http://bugs.python.org/issue15881#msg170215
import glob

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

subpackages = [x.replace('/', '.') for x in glob.glob('tripaille/commands/*') if not x.endswith('.py')]
subpackages += [x.replace('/', '.') for x in glob.glob('tripal/*') if not x.endswith('.py')]

setup(
    name="tripal",
    version='2.0.4',
    description="Tripal library",
    author="Anthony Bretaudeau",
    author_email="anthony.bretaudeau@inra.fr",
    url="https://github.com/galaxy-genome-annotation/python-tripal",
    install_requires=['requests>=2.4.3', 'wrapt', 'click', 'pyyaml', 'future'],
    packages=[
        'tripal',
        'tripaille',
        'tripaille.commands',
    ] + subpackages,
    license='MIT',
    platforms="Posix; MacOS X; Windows",
    entry_points='''
        [console_scripts]
        tripaille=tripaille.cli:tripaille
    ''',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7"
    ])



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from collections import OrderedDict
from datetime import datetime
import logging
import os
from tripal.client import Client

logging.getLogger("requests").setLevel(logging.CRITICAL)
log = logging.getLogger()


class AnalysisClient(Client):
    CLIENT_BASE = '/tripal_api/'

    def get_analysis_nodes(self, node=None):
        """
        Get analysis nodes

        :type node: int
        :param node: filter on node id

        :rtype: list of dict
        :return: Analysis node information
        """

        if node:
            nodes = [self._get('node/%s' % node, {})]
        else:
            nodes = self._get('node', {})

        nodes = [n for n in nodes if n['type'].startswith('chado_analysis')]

        return nodes

    def get_analyses(self, analysis_id=None, name=None, program=None, programversion=None,
                     algorithm=None, sourcename=None, sourceversion=None,
                     sourceuri=None, date_executed=None):
        """
        Get analyses

        :type analysis_id: str
        :param analysis_id: An analysis ID

        :type name: str
        :param name: analysis name

        :type program: str
        :param program: analysis program

        :type programversion: str
        :param programversion: analysis programversion

        :type algorithm: str
        :param algorithm: analysis algorithm

        :type sourcename: str
        :param sourcename: analysis sourcename

        :type sourceversion: str
        :param sourceversion: analysis sourceversion

        :type sourceuri: str
        :param sourceuri: analysis sourceuri

        :type date_executed: str
        :param date_executed: analysis date_executed (yyyy-mm-dd)

        :rtype: list of dict
        :return: Analysis information
        """

        ans = self._request('chado/list', {'table': 'analysis'})

        if analysis_id:
            ans = [v for v in ans if v['analysis_id'] == str(analysis_id)]
        if name:
            ans = [v for v in ans if v['name'] == name]
        if program:
            ans = [v for v in ans if v['program'] == program]
        if programversion:
            ans = [v for v in ans if v['programversion'] == programversion]
        if algorithm:
            ans = [v for v in ans if v['algorithm'] == algorithm]
        if sourcename:
            ans = [v for v in ans if v['sourcename'] == sourcename]
        if sourceversion:
            ans = [v for v in ans if v['sourceversion'] == sourceversion]
        if sourceuri:
            ans = [v for v in ans if v['sourceuri'] == sourceuri]
        if date_executed:
            ans = [v for v in ans if v['timeexecuted'].startswith(date_executed)]

        return ans

    def add_analysis(self, name, program, programversion, sourcename, algorithm="",
                     sourceversion="", sourceuri="", description="", date_executed=None):
        """
        Create an analysis

        :type name: str
        :param name: analysis name

        :type program: str
        :param program: analysis program

        :type programversion: str
        :param programversion: analysis programversion

        :type algorithm: str
        :param algorithm: analysis algorithm

        :type sourcename: str
        :param sourcename: analysis sourcename

        :type sourceversion: str
        :param sourceversion: analysis sourceversion

        :type sourceuri: str
        :param sourceuri: analysis sourceuri

        :type description: str
        :param description: analysis description

        :type date_executed: str
        :param date_executed: analysis date_executed (yyyy-mm-dd)

        :rtype: dict
        :return: Analysis information
        """

        date = datetime.today()
        if date_executed:
            date = datetime.strptime(date_executed, '%Y-%m-%d')

        params = {
            'type': 'chado_analysis',
            'analysisname': name,
            'program': program,
            'programversion': programversion,
            'algorithm': algorithm,
            'sourcename': sourcename,
            'sourceversion': sourceversion,
            'sourceuri': sourceuri,
            'description': description,
            'timeexecuted': {
                'day': date.strftime('%d'),
                'month': date.strftime('%m'),
                'year': date.strftime('%Y')
            },
        }

        return self._request('node', params)

    def load_blast(self, name, program, programversion, sourcename, blast_output,
                   blast_ext=None, blastdb=None, blastdb_id=None,
                   blast_parameters=None, query_re=None, query_type=None,
                   query_uniquename=False, is_concat=False, search_keywords=False,
                   no_wait=False, algorithm="", sourceversion="", sourceuri="",
                   description="", date_executed=None):
        """
        Create a Blast analysis

        :type name: str
        :param name: analysis name

        :type program: str
        :param program: analysis program

        :type programversion: str
        :param programversion: analysis programversion

        :type sourcename: str
        :param sourcename: analysis sourcename

        :type blast_output: str
        :param blast_output: Path to the Blast file to load (single XML file, or directory containing multiple XML files)

        :type blast_ext: str
        :param blast_ext: If looking for files in a directory, extension of the blast result files

        :type blastdb: str
        :param blastdb: Name of the database blasted against (must be in the Chado db table)

        :type blastdb_id: str
        :param blastdb_id: ID of the database blasted against (must be in the Chado db table)

        :type blast_parameters: str
        :param blast_parameters: Blast parameters used to produce these results

        :type query_re: str
        :param query_re: The regular expression that can uniquely identify the query name. This parameters is required if the feature name is not the first word in the blast query name.

        :type query_type: str
        :param query_type: The feature type (e.g. \'gene\', \'mRNA\', \'contig\') of the query. It must be a valid Sequence Ontology term.

        :type query_uniquename: bool
        :param query_uniquename: Use this if the --query-re regular expression matches unique names instead of names in the database.

        :type is_concat: bool
        :param is_concat: If the blast result file is simply a list of concatenated blast results.

        :type search_keywords: bool
        :param search_keywords: Extract keywords for Tripal search

        :type no_wait: bool
        :param no_wait: Do not wait for job to complete

        :type algorithm: str
        :param algorithm: analysis algorithm

        :type sourceversion: str
        :param sourceversion: analysis sourceversion

        :type sourceuri: str
        :param sourceuri: analysis sourceuri

        :type description: str
        :param description: analysis description

        :type date_executed: str
        :param date_executed: analysis date_executed (yyyy-mm-dd)

        :rtype: str
        :return: Loading information
        """

        if blastdb_id:
            found_db = self.tripal.db.get_dbs(db_id=blastdb_id)
            if not found_db:
                raise Exception("Invalid db ID")
        elif blastdb:
            found_db = self.tripal.db.get_dbs(name=blastdb)
            if not found_db:
                raise Exception("Invalid db name")

            blastdb_id = found_db[0]['db_id']

        if not blastdb_id:
            raise Exception("Either blastdb or blastdb_id is required")

        date = datetime.today()
        if date_executed:
            date = datetime.strptime(date_executed, '%Y-%m-%d')

        params = {
            'type': 'chado_analysis_blast',
            'analysisname': name,
            'program': program,
            'programversion': programversion,
            'algorithm': algorithm,
            'sourcename': sourcename,
            'sourceversion': sourceversion,
            'sourceuri': sourceuri,
            'description': description,
            'timeexecuted': {
                'day': date.strftime('%d'),
                'month': date.strftime('%m'),
                'year': date.strftime('%Y')
            },

            'blastdb': blastdb_id,
            'blastfile': blast_output,
            'blastfile_ext': blast_ext,
            'blastjob': 1,  # no reason to not launch a job
            'blastparameters': blast_parameters,
            'query_re': query_re,
            'query_type': query_type,
            'query_uniquename': query_uniquename,
            'is_concat': int(is_concat),
            'search_keywords': int(search_keywords),
        }

        res = self._request('node', params)

        an_node = self.get_analysis_nodes(res['nid'])

        if not an_node:
            raise Exception("Could not find analysis node with id %s" % res['nid'])

        an_node = an_node[0]

        if no_wait:
            return an_node
        else:
            an_id = an_node['analysis']['analysis_id']
            job_id = None
            jobs = self.tripal.job.get_jobs()
            for job in jobs:
                if job['modulename'] == 'tripal_analysis_blast' and job['raw_arguments'][0] == an_id:
                    job_id = job['job_id']

            if not job_id:
                raise Exception("Could not get job id for analysis %s" % an_id)

            return self._run_job_and_wait(job_id)

    def load_interpro(self, name, program, programversion, sourcename,
                      interpro_output, interpro_parameters=None, query_re=None,
                      query_type=None, query_uniquename=False, parse_go=False,
                      no_wait=False, algorithm="", sourceversion="", sourceuri="",
                      description="", date_executed=None):
        """
        Create an Interpro analysis

        :type name: str
        :param name: analysis name

        :type program: str
        :param program: analysis program

        :type programversion: str
        :param programversion: analysis programversion

        :type sourcename: str
        :param sourcename: analysis sourcename

        :type interpro_output: str
        :param interpro_output: Path to the InterProScan file to load (single XML file, or directory containing multiple XML files)

        :type interpro_parameters: str
        :param interpro_parameters: InterProScan parameters used to produce these results

        :type query_re: str
        :param query_re: The regular expression that can uniquely identify the query name. This parameters is required if the feature name is not the first word in the blast query name.

        :type query_type: str
        :param query_type: The feature type (e.g. \'gene\', \'mRNA\', \'contig\') of the query. It must be a valid Sequence Ontology term.

        :type query_uniquename: bool
        :param query_uniquename: Use this if the query_re regular expression matches unique names instead of names in the database.

        :type parse_go: bool
        :param parse_go: Load GO annotation to the database

        :type no_wait: bool
        :param no_wait: Do not wait for job to complete

        :type algorithm: str
        :param algorithm: analysis algorithm

        :type sourceversion: str
        :param sourceversion: analysis sourceversion

        :type sourceuri: str
        :param sourceuri: analysis sourceuri

        :type description: str
        :param description: analysis description

        :type date_executed: str
        :param date_executed: analysis date_executed (yyyy-mm-dd)

        :rtype: str
        :return: Loading information
        """

        date = datetime.today()
        if date_executed:
            date = datetime.strptime(date_executed, '%Y-%m-%d')

        params = {
            'type': 'chado_analysis_interpro',
            'analysisname': name,
            'program': program,
            'programversion': programversion,
            'algorithm': algorithm,
            'sourcename': sourcename,
            'sourceversion': sourceversion,
            'sourceuri': sourceuri,
            'description': description,
            'timeexecuted': {
                'day': date.strftime('%d'),
                'month': date.strftime('%m'),
                'year': date.strftime('%Y')
            },

            'interprofile': interpro_output,
            'interprojob': 1,
            'parsego': int(parse_go),
            'interproparameters': interpro_parameters,
            'query_re': query_re,
            'query_type': query_type,
            'query_uniquename': query_uniquename,
        }

        res = self._request('node', params)

        an_node = self.get_analysis_nodes(res['nid'])

        if not an_node:
            raise Exception("Could not find analysis node with id %s" % res['nid'])

        an_node = an_node[0]

        if no_wait:
            return an_node
        else:
            an_id = an_node['analysis']['analysis_id']
            job_id = None
            jobs = self.tripal.job.get_jobs()
            for job in jobs:
                if job['modulename'] == 'tripal_analysis_interpro' and job['raw_arguments'][0] == an_id:
                    job_id = job['job_id']

            if not job_id:
                raise Exception("Could not get job id for analysis %s" % an_id)

            return self._run_job_and_wait(job_id)

    def load_go(self, name, program, programversion, sourcename, gaf_output,
                gaf_ext=None, query_type=None, query_uniquename=False,
                method='add', re_name=None, no_wait=False, algorithm="",
                sourceversion="", sourceuri="", description="", date_executed=None):
        """
        Create a GO analysis

        :type name: str
        :param name: analysis name

        :type program: str
        :param program: analysis program

        :type programversion: str
        :param programversion: analysis programversion

        :type sourcename: str
        :param sourcename: analysis sourcename

        :type gaf_output: str
        :param gaf_output: Path to the GAF file to load (single file, or directory containing multiple GAF files)

        :type gaf_ext: str
        :param gaf_ext: If looking for files in a directory, extension of the GAF files

        :type query_type: str
        :param query_type: The feature type (e.g. \'gene\', \'mRNA\', \'contig\') of the query. It must be a valid Sequence Ontology term.

        :type query_uniquename: bool
        :param query_uniquename: Use this if the --query-re regular expression matches unique names instead of names in the database.

        :type method: str
        :param method: Import method ('add' or 'remove')

        :type re_name: str
        :param re_name: Regular expression to extract the feature name from GAF file.

        :type no_wait: bool
        :param no_wait: Do not wait for job to complete

        :type algorithm: str
        :param algorithm: analysis algorithm

        :type sourceversion: str
        :param sourceversion: analysis sourceversion

        :type sourceuri: str
        :param sourceuri: analysis sourceuri

        :type description: str
        :param description: analysis description

        :type date_executed: str
        :param date_executed: analysis date_executed (yyyy-mm-dd)

        :rtype: str
        :return: Loading information
        """

        methods = {
            'add': 1,
            'remove': 2,
        }

        if method not in methods:
            raise Exception("Method should be 'add' or 'remove'")

        date = datetime.today()
        if date_executed:
            date = datetime.strptime(date_executed, '%Y-%m-%d')

        params = {
            'type': 'chado_analysis_go',
            'analysisname': name,
            'program': program,
            'programversion': programversion,
            'algorithm': algorithm,
            'sourcename': sourcename,
            'sourceversion': sourceversion,
            'sourceuri': sourceuri,
            'description': description,
            'timeexecuted': {
                'day': date.strftime('%d'),
                'month': date.strftime('%m'),
                'year': date.strftime('%Y')
            },

            'gaf_file': gaf_output,
            'gaf_file_ext': gaf_ext,
            'seq_type': query_type,
            'query_uniquename': query_uniquename,
            'method': methods[method],
            're_name': re_name,
            'gojob': 1,
        }

        res = self._request('node', params)

        an_node = self.get_analysis_nodes(res['nid'])

        if not an_node:
            raise Exception("Could not find analysis node with id %s" % res['nid'])

        an_node = an_node[0]

        if no_wait:
            return an_node
        else:
            an_id = an_node['analysis']['analysis_id']
            job_id = None
            jobs = self.tripal.job.get_jobs()
            for job in jobs:
                if job['modulename'] == 'tripal_analysis_go' and job['raw_arguments'][0] == an_id:
                    job_id = job['job_id']

            if not job_id:
                raise Exception("Could not get job id for analysis %s" % an_id)

            return self._run_job_and_wait(job_id)

    def load_fasta(self, fasta, organism=None, organism_id=None, analysis=None,
                   analysis_id=None, sequence_type="contig", re_name='',
                   re_uniquename='', db_ext_id='', re_accession='',
                   rel_type='', rel_subject_re='', rel_subject_type='',
                   method='insup', match_type='uniquename', job_name=None, no_wait=False):
        """
        Load fasta sequences

        :type fasta: str
        :param fasta: Path to the Fasta file to load

        :type organism: str
        :param organism: Organism common name or abbreviation

        :type organism_id: int
        :param organism_id: Organism ID

        :type analysis: str
        :param analysis: Analysis name

        :type analysis_id: int
        :param analysis_id: Analysis ID

        :type sequence_type: str
        :param sequence_type: Sequence type

        :type re_name: str
        :param re_name: Regular expression for the name

        :type re_uniquename: str
        :param re_uniquename: Regular expression for the unique name

        :type db_ext_id: str
        :param db_ext_id: External DB ID

        :type re_accession: str
        :param re_accession: Regular expression for the accession from external DB

        :type rel_type: str
        :param rel_type: Relation type (part_of or derives_from)

        :type rel_subject_re: str
        :param rel_subject_re: Relation subject regular expression (used to extract id of related entity)

        :type rel_subject_type: str
        :param rel_subject_type: Relation subject type (must match already loaded data, e.g. mRNA)

        :type method: str
        :param method: Insertion method (insert, update or insup, default=insup (Insert and Update))

        :type match_type: str
        :param match_type: Match type for already loaded features (name or uniquename; default=uniquename; used for "Update only" or "Insert and update" methods)'

        :type job_name: str
        :param job_name: Name of the job

        :type no_wait: bool
        :param no_wait: Do not wait for job to complete

        :rtype: str
        :return: Loading information
        """

        if organism_id:
            found_org = self.tripal.organism.get_organisms(organism_id=organism_id)
            if not found_org:
                raise Exception("Invalid organism ID")
        elif organism:
            found_org = self.tripal.organism.get_organisms(common=organism)
            if not found_org:
                found_org = self.tripal.organism.get_organisms(abbr=organism)

            if not found_org:
                raise Exception("Invalid organism name")

            organism_id = found_org[0]['organism_id']

        if not organism_id:
            raise Exception("Either organism or organism_id is required")

        if analysis_id:
            found_an = self.get_analyses(analysis_id=analysis_id)
            if not found_an:
                raise Exception("Invalid analysis ID")
        elif analysis:
            found_an = self.get_analyses(name=analysis)

            if not found_an:
                raise Exception("Invalid analysis name")

            analysis_id = found_an[0]['analysis_id']

        if not analysis_id:
            raise Exception("Either analysis or analysis_id is required")

        methods = {
            'insert': 'Insert only',
            'update': 'Update only',
            'insup': 'Insert and update',
        }

        if method not in methods:
            raise Exception("Method should be 'insert', 'update', or 'insup'")

        rel_types = {
            'part_of': 'part_of',
            'derives_from': 'derives_from',
        }

        if rel_type and rel_type not in rel_types:
            raise Exception("rel_type should be 'part_of' or 'derives_from'")
        elif rel_type:
            rel_type = rel_types[rel_type]

        match_types = {
            'name': 'Name',
            'uniquename': 'Unique name',
        }

        if match_type not in match_types:
            raise Exception("match_type should be 'name' or 'uniquename'")

        if not job_name:
            job_name = 'Import FASTA file: %s' % os.path.basename(fasta)

        uid = 1  # user id is not really used by the loader, 1 is admin user

        job_args = [fasta, organism_id, sequence_type, re_name, re_uniquename, re_accession,
                    db_ext_id, rel_type, rel_subject_re, rel_subject_type,
                    methods[method], uid, analysis_id, match_types[match_type]]

        r = self.tripal.job.add_job(job_name, 'tripal_feature', 'tripal_feature_load_fasta', job_args)
        if 'job_id' not in r or not r['job_id']:
            raise Exception("Failed to create job, received %s" % r)

        if no_wait:
            return r
        else:
            return self._run_job_and_wait(r['job_id'])

    def load_gff3(self, gff, organism=None, organism_id=None, analysis=None,
                  analysis_id=None, import_mode='update', target_organism=None,
                  target_organism_id=None, target_type=None, target_create=False,
                  start_line=None, landmark_type=None, alt_id_attr=None,
                  create_organism=None, re_mrna=None, re_protein=None, job_name=None, no_wait=False):
        """
        Load GFF3 file

        :type gff: str
        :param gff: Path to the GFF file to load

        :type organism: str
        :param organism: Organism common name or abbreviation

        :type organism_id: int
        :param organism_id: Organism ID

        :type analysis: str
        :param analysis: Analysis name

        :type analysis_id: int
        :param analysis_id: Analysis ID

        :type import_mode: str
        :param import_mode: Import mode (add_only=existing features won't be touched, update=existing features will be updated and obsolete attributes kept, refresh=existing features will be updated and obsolete attributes removed, remove=features present in the db and in the GFF3 file will be removed)')

        :type target_organism: str
        :param target_organism: In case of Target attribute in the GFF3, choose the organism abbreviation or common name to which target sequences belong. Select this only if target sequences belong to a different organism than the one specified with --organism-id. And only choose an organism here if all of the target sequences belong to the same species. If the targets in the GFF file belong to multiple different species then the organism must be specified using the 'target_organism=genus:species' attribute in the GFF file.')

        :type target_organism_id: int
        :param target_organism_id: In case of Target attribute in the GFF3, choose the organism ID to which target sequences belong. Select this only if target sequences belong to a different organism than the one specified with --organism-id. And only choose an organism here if all of the target sequences belong to the same species. If the targets in the GFF file belong to multiple different species then the organism must be specified using the \'target_organism=genus:species\' attribute in the GFF file.')

        :type target_type: str
        :param target_type: In case of Target attribute in the GFF3, if the unique name for a target sequence is not unique (e.g. a protein and an mRNA have the same name) then you must specify the type for all targets in the GFF file. If the targets are of different types then the type must be specified using the \'target_type=type\' attribute in the GFF file. This must be a valid Sequence Ontology (SO) term.')

        :type target_create: bool
        :param target_create: In case of Target attribute in the GFF3, if the target feature cannot be found, create one using the organism and type specified above, or using the \'target_organism\' and \'target_type\' fields specified in the GFF file. Values specified in the GFF file take precedence over those specified above.')

        :type start_line: int
        :param start_line: The line in the GFF file where importing should start

        :type landmark_type: str
        :param landmark_type: A Sequence Ontology type for the landmark sequences in the GFF fie (e.g. \'chromosome\').

        :type alt_id_attr: str
        :param alt_id_attr: When ID attribute is absent, specify which other attribute can uniquely identify the feature.

        :type create_organism: bool
        :param create_organism: Create organisms when encountering organism attribute (these lines will be skip otherwise)

        :type re_mrna: str
        :param re_mrna: Regular expression for the mRNA name

        :type re_protein: str
        :param re_protein: Replacement string for the protein name

        :type job_name: str
        :param job_name: Name of the job

        :type no_wait: bool
        :param no_wait: Do not wait for job to complete

        :rtype: str
        :return: Loading information
        """

        if organism_id:
            found_org = self.tripal.organism.get_organisms(organism_id=organism_id)
            if not found_org:
                raise Exception("Invalid organism ID")
        elif organism:
            found_org = self.tripal.organism.get_organisms(common=organism)
            if not found_org:
                found_org = self.tripal.organism.get_organisms(abbr=organism)

            if not found_org:
                raise Exception("Invalid organism name")

            organism_id = found_org[0]['organism_id']

        if not organism_id:
            raise Exception("Either organism or organism_id is required")

        if analysis_id:
            found_an = self.get_analyses(analysis_id=analysis_id)
            if not found_an:
                raise Exception("Invalid analysis ID")
        elif analysis:
            found_an = self.get_analyses(name=analysis)

            if not found_an:
                raise Exception("Invalid analysis name")

            analysis_id = found_an[0]['analysis_id']

        if not analysis_id:
            raise Exception("Either analysis or analysis_id is required")

        if target_organism_id:
            found_torg = self.tripal.organism.get_organisms(organism_id=target_organism_id)
            if not found_torg:
                raise Exception("Invalid target organism ID")
        elif target_organism:
            found_torg = self.tripal.organism.get_organisms(common=target_organism)
            if not found_torg:
                found_torg = self.tripal.organism.get_organisms(abbr=target_organism)

            if not found_torg:
                raise Exception("Invalid target organism name")

            target_organism_id = found_torg[0]['organism_id']

        import_modes = {
            'add_only': 'add_only',
            'update': 'update',
            'refresh': 'refresh',
            'remove': 'remove',
        }

        if import_mode not in import_modes:
            raise Exception("import_mode should be 'add_only', 'update', 'refresh' or 'remove'")

        if not job_name:
            job_name = 'Import GFF3 file: %s' % os.path.basename(gff)

        transaction = 1  # use transaction or not, no reason to disable this

        job_args = [gff, organism_id, analysis_id, int(import_mode == 'add_only'),
                    int(import_mode == 'update'), int(import_mode == 'refresh'), int(import_mode == 'remove'),
                    transaction, target_organism_id, target_type, int(target_create), start_line,
                    landmark_type, alt_id_attr, int(create_organism), re_mrna, re_protein]

        r = self.tripal.job.add_job(job_name, 'tripal_feature', 'tripal_feature_load_gff3', job_args)
        if 'job_id' not in r or not r['job_id']:
            raise Exception("Failed to create job, received %s" % r)

        if no_wait:
            return r
        else:
            return self._run_job_and_wait(r['job_id'])

    def sync(self, analysis=None, analysis_id=None, job_name=None, no_wait=None):
        """
        Synchronize an analysis

        :type analysis: str
        :param analysis: Analysis name

        :type analysis_id: str
        :param analysis_id: ID of the analysis to sync

        :type job_name: str
        :param job_name: Name of the job

        :type no_wait: bool
        :param no_wait: Return immediately without waiting for job completion

        :rtype: str
        :return: status
        """

        if analysis_id:
            found_an = self.get_analyses(analysis_id=analysis_id)
            if not found_an:
                raise Exception("Invalid analysis ID")
        elif analysis:
            found_an = self.get_analyses(name=analysis)
            if not found_an:
                raise Exception("Invalid analysis name")

            analysis_id = found_an[0]['analysis_id']

        if not analysis_id:
            raise Exception("Either analysis or analysis_id is required")

        if not job_name:
            job_name = 'Sync Analysis'

        job_args = OrderedDict()
        job_args['base_table'] = 'analysis'
        job_args['max_sync'] = ''
        job_args['organism_id'] = ''
        job_args['types'] = []
        job_args['ids'] = [int(analysis_id)]
        job_args['linking_table'] = 'chado_analysis'
        job_args['node_type'] = 'chado_analysis'

        r = self.tripal.job.add_job(job_name, 'chado_feature', 'chado_node_sync_records', job_args)
        if 'job_id' not in r or not r['job_id']:
            raise Exception("Failed to create job, received %s" % r)

        if no_wait:
            return r
        else:
            return self._run_job_and_wait(r['job_id'])



import click
from tripaille.commands.db.get_dbs import cli as func0
from tripaille.commands.db.get_mviews import cli as func1
from tripaille.commands.db.index import cli as func2
from tripaille.commands.db.populate_mviews import cli as func3


@click.group()
def cli():
    pass


cli.add_command(func0)
cli.add_command(func1)
cli.add_command(func2)
cli.add_command(func3)
